import { FormState } from "./form";
import React from "react";
export declare type ElementProps<C extends React.FunctionComponent<any> | keyof JSX.IntrinsicElements> = C extends React.FunctionComponent<infer P> ? P : C extends keyof JSX.IntrinsicElements ? JSX.IntrinsicElements[C] : never;
export declare type FieldProps<T extends object, K extends keyof T, C> = {
    /**
     * The form to make a field for
     */
    form: FormState<T>;
    /**
     * The name of the field in form
     */
    name: K;
    /**
     * The element to create, this can be a string specifying "input", "select", "textarea" or a custom component. Is "input" by default. The props of the passed custom component are available on this field.
     *
     * **Examples**
     *
     * `<Field as="textarea" rows={10} />`
     *
     * `<Field as={CustomInput} {...} />`
     */
    as?: C;
    /**
     * Wheter to hide this field if is set to undefined or null.
     */
    hideWhenNull?: boolean;
    /**
     * A serializer is a function that converts the field value to a string/boolean value that can be passed to input. Leave undefined to use the default serializer.
     *
     * You can change the behaviour of the default serializer using the `type` prop.
     */
    serializer?: Serializer<T[K]>;
    /**
     * A deserializer is a function that converts the string/boolean input value to the real form value. Leave undefined to use the default deserializer.
     *
     * You can change the behaviour of the default deserializer using the `type` prop.
     */
    deserializer?: Deserializer<T[K]>;
    /**
     * The class to set when there is an error on this field.
     */
    errorClassName?: string;
    /**
     * The class to set when this field has been modified.
     */
    dirtyClassName?: string;
    /**
     * The style to set when where is an error on this field.
     */
    errorStyle?: React.CSSProperties;
    /**
     * The style to set when this field has been modified.
     */
    dirtyStyle?: React.CSSProperties;
    /**
     * The ref to pass to your input component.
     */
    innerRef?: React.Ref<any>;
};
export declare function Field<T extends object, K extends keyof T, C extends React.FunctionComponent<any> | keyof JSX.IntrinsicElements = "input">(props: FieldProps<T, K, C> & Omit<ElementProps<C>, "value" | "checked" | "onChange" | "field" | "ref" | keyof FieldProps<T, K, C> | keyof SerializeProps> & SerializeProps<T[K]>): React.ReactElement<any, string | React.JSXElementConstructor<any>> | null;
export declare type SerializeType = "number" | "text" | "password" | "date" | "datetime-local" | "radio" | "checkbox" | "color" | "email" | "text" | "month" | "url" | "week" | "time" | "tel" | "range";
export declare type Serializer<T> = (currentValue: T, props: SerializeProps<T>) => boolean | string;
export declare type Deserializer<T> = (inputValue: string | boolean, currentValue: T, props: SerializeProps<T>) => T;
export declare type SerializeProps<V = any> = {
    dateAsNumber?: boolean;
    setUndefinedOnUncheck?: boolean;
    setNullOnUncheck?: boolean;
    type?: SerializeType;
    value?: V extends any[] ? V[number] : V;
};
export declare function defaultSerializer<T>(currentValue: T, props: SerializeProps<T>): boolean | string;
export declare function defaultDeserializer<T>(inputValue: string | boolean, currentValue: T, props: SerializeProps<T>): any;
