/// <reference types="react" />
export declare type ListenerCallback = () => void;
export declare type ListenerMap = {
    [T in string]?: ListenerCallback;
};
export declare type Validator<T, Error> = (values: T) => ErrorType<T, Error> | undefined | Promise<ErrorType<T, Error> | undefined>;
export declare type ChildFormMap<T extends object, State, Error extends string> = {
    [K in KeysOfType<T, object>]?: ChildFormState<T, K, State, Error>;
};
export declare type DirtyMap<T> = {
    [Key in keyof T]?: boolean;
};
export declare type ErrorType<T, Error> = Error | (T extends {} ? ErrorMap<NonNullable<T>, Error> : never);
export declare type ErrorMap<T, Error> = {
    [Key in keyof T]?: ErrorType<T[Key], Error>;
};
export declare type DefaultError = string;
export declare type DefaultState = {
    isSubmitting: boolean;
};
export declare type FieldsOfType<T, Field> = {
    [Key in keyof T as NonNullable<T[Key]> extends Field ? Key : never]: T[Key];
};
export declare type KeysOfType<T extends FieldsOfType<any, Field>, Field> = keyof FieldsOfType<T, Field>;
export declare class FormState<T extends object, State = DefaultState, Error extends string = DefaultError> {
    /**
     * The id of this form, for debugging purposes.
     */
    readonly formId: number;
    /**
     * The form's validator.
     */
    validator?: Validator<T, Error>;
    /**
     * Should the form validate on each value change?
     */
    validateOnChange: boolean;
    /**
     * Should the form validate when it gets mounted?
     */
    validateOnMount: boolean;
    /**
     * The values on this form. Use `setValues()` to set these.
     */
    readonly values: T;
    /**
     * The default values on this form. Use `setValues(?,?,true)` to set these.
     */
    readonly defaultValues: T;
    /**
     * The dictionary that maps object fields to child forms.
     */
    readonly childMap: ChildFormMap<T, State, Error>;
    /**
     * The dictionary that contains dirty states for each field.
     */
    readonly dirtyMap: DirtyMap<T>;
    /**
     * The dictionary that contains errors for each field.
     */
    readonly errorMap: ErrorMap<T, Error>;
    private _state;
    private listeners;
    private anyListeners;
    private counter;
    private static formCounter;
    constructor(values: T, defaultValues: T, defaultState: State, validator: Validator<T, Error> | undefined, validateOnMount: boolean, validateOnChange: boolean);
    /**
     * Gets the state of the current form.
     */
    get state(): State;
    /**
     * Is this form modified?
     */
    get dirty(): boolean;
    /**
     * Does this form contain any error?
     */
    get error(): boolean;
    /**
     * Sets a value without calculating whether is has been modified.
     * @param dirty Is this field dirty? Leave undefined to not set any dirty value. (can always be overridden by child forms)
     * @param validate Should the form validate after value set? Overrides `validateOnChange`.
     * @param isDefault Is this the default value for the said field?
     * @param notifyChild Should this form notify any child form about the change?
     * @param notifyParent Should this form notify any parent form about the change?
     * @param setValuesWasUsed Fire all `anyListeners` after field is set? You should not touch this. (will be false for bulk sets, they will call fireAnyListeners() after every field is set)
     */
    setValueInternal<Key extends keyof T>(key: Key, value: T[Key] | undefined, dirty: boolean, validate?: boolean, isDefault?: boolean, notifyChild?: boolean, notifyParent?: boolean, fireAny?: boolean): void;
    /**
     * Set a value on this form.
     * @param key The field to set.
     * @param value The field's new value.
     * @param validate Should the form validate?
     * @param isDefault Is this the default value?
     * @param notifyChild Should this form notify the child form about this change?
     * @param notifyParent Should this form notify the parent form about this change?
     * @param fireAny Fire all `anyListeners` after field is set?
     */
    setValue<Key extends keyof T>(key: Key, value: T[Key] | undefined, validate?: boolean, isDefault?: boolean, notifyChild?: boolean, notifyParent?: boolean, fireAny?: boolean): void;
    /**
     * Set multiple values OR default values on this form.
     * @param values The new values to set on this form.
     * @param validate Validate? Overrides `validateOnChange`.
     * @param isDefault Leave undefined to set both `values` and `defaultValues`. Set to true to only set `defaultValues` and false to only set `values`.
     * @param notifyChild Should this form notify the child form about this change?
     * @param notifyParent Should this form notify the parent form about this change?
     */
    setValues(values: Partial<T> | undefined, validate?: boolean, isDefault?: boolean, notifyChild?: boolean, notifyParent?: boolean): void;
    /**
     * Force validation on this form. Required when `validateOnChange` is disabled. **This function works with both asynchronous and synchronous validators.**
     * @returns true if the form is valid.
     */
    validate(): Promise<boolean>;
    /**
     * Force validation on this form. Required when `validateOnChange` is disabled. **This only works if you have a synchronous validator set (not async).**
     * @returns true if the form is valid.
     */
    validateSync(): boolean;
    /**
     * Sets an error on this form
     * @param key The field to set an error on.
     * @param error The error.
     * @param notifyChild Should this form notify the child form about this change?
     * @param notifyParent Should this form notify the parent form about this change?
     * @param fireAny Fire all `anyListeners` after field is set?
     */
    setError<Key extends keyof T>(key: Key, error: ErrorType<NonNullable<T[Key]>, Error> | undefined, notifyChild?: boolean, notifyParent?: boolean, fireAny?: boolean): boolean;
    /**
     * Sets all the errors on this form.
     * @param errors The new errors for this form. Use {} to clear errors. **The format of this error object must follow the same structure of the values object, but each value is replaced by its error.**
     * @param notifyChild Should this form notify the child form about this change?
     * @param notifyParent Should this form notify the parent form about this change?
     */
    setErrors(errors: ErrorType<T, Error>, notifyChild?: boolean, notifyParent?: boolean): boolean;
    /**
     * Reset this form's values to the default values.
     * @param notifyChild Should this form notify the child form about this change?
     * @param notifyParent Should this form notify the parent form about this change?
     */
    resetAll(validate?: boolean, notifyChild?: boolean, notifyParent?: boolean): void;
    /**
     * Reset a form's field to its default value.
     * @param key The field to reset.
     * @param notifyChild Should this form notify the child form about this change?
     * @param notifyParent Should this form notify the parent form about this change?
     */
    reset(key: keyof T, validate?: boolean, notifyChild?: boolean, notifyParent?: boolean): void;
    /**
     * Sets the state for this form, and also on child and parent forms by default.
     * @param newState The new form state.
     * @param notifyChild Set the state on the child too?
     * @param notifyParent Set the state on the parent too?
     */
    setState(newState: State, notifyChild?: boolean, notifyParent?: boolean): void;
    /**
     * Creates a submit handler to pass to your `<form onSubmit={...}>`. The function executes the passed handler only if the form validates correctly.
     * @param handler The handler to execute when this form contains no errors.
     */
    handleSubmit(handler: (form: FormState<T, State, Error>, ev: React.FormEvent<HTMLFormElement>) => void | Promise<void>): (ev: import("react").FormEvent<HTMLFormElement>) => Promise<void>;
    /**
     * Listen for changes on a field, will trigger when value, defaultValue, dirty and error changes for a field. Make sure you pass its return value back to `ignore()` after you are done listening.
     * @param key The field to listen to.
     * @param listener Change callback.
     */
    listen(key: keyof T, listener: ListenerCallback): string;
    /**
     * Listen for any change on this form. Make sure you pass its return value back to `ignoreAny()` after you are done listening.
     * @param listener Change callback.
     */
    listenAny(listener: ListenerCallback): string;
    /**
     * Ignore changes on a field.
     * @param key The field to ignore.
     * @param id The callback to ignore.
     */
    ignore(key: keyof T, id: string): void;
    /**
     * Ignore changes on this form.
     * @param id The callback to ignore.
     */
    ignoreAny(id: string): void;
    protected fireListeners(key: keyof T): void;
    protected fireAnyListeners(): void;
}
export declare class ChildFormState<T extends FieldsOfType<any, object>, K extends KeysOfType<T, object>, State, Error extends string> extends FormState<NonNullable<T[K]>, State, Error> {
    name: K;
    readonly parent: FormState<T, State, Error>;
    constructor(parent: FormState<T, State, Error>, name: K);
}
