import { DefaultState, DefaultError, FormState, ChildFormState, Validator, FieldsOfType, KeysOfType, ErrorType } from "./form";
/**
 * Creates a new root form.
 * This hook doesn't cause a rerender.
 * @param defaultValues The default values for this form. When this value changes, nothing happens, use useEffect() with form.setValues to set form values on state change.
 * @param validator The validator to use, optional.
 * @param validateOnChange Validate on change? Optional, default is false.
 * @param validateOnMount Validate on mount? Optional, default is false.
 * @param defaultState The default state for this form. Form state contains custom global states, example: isSubmitting, isLoading ... Optional, default is `{ isSubmitting: false }`.
 */
export declare function useForm<T extends object, State = DefaultState, Error extends string = DefaultError>(defaultValues: T, validator?: Validator<T, Error>, validateOnChange?: boolean, validateOnMount?: boolean, defaultState?: State): FormState<T, State, Error>;
/**
 * Creates a nested form for another root or nested form. You must use this for object and array (see useArrayField) field.
 * This hook doesn't cause a rerender.
 * @param parentForm The parent form.
 * @param name The parent's field to create a child form for.
 */
export declare function useObjectField<T extends FieldsOfType<any, object | any[]>, K extends KeysOfType<T, object | any[]>, State = DefaultState, Error extends string = DefaultError>(parentForm: FormState<T, State, Error>, name: K): ChildFormState<T, K, State, Error>;
export interface FormField<T extends object = any, K extends keyof T = never, State extends DefaultState = DefaultState, Error extends string = DefaultError> {
    value: T[K];
    defaultValue: T[K];
    setValue: (value: T[K]) => void;
    dirty: boolean;
    error: ErrorType<T[K], Error> | undefined;
    state: State;
    form: FormState<T, State, Error>;
}
/**
 * Listen for changes on a form's field. Behaves like useState.
 * You shouldn't use this hook in large components, as it rerenders each time something changes. Use the wrapper <Listener /> instead.
 * @param form The form to listen on.
 * @param name The form's field to listen to.
 */
export declare function useListener<T extends object, K extends keyof T, State extends DefaultState = DefaultState, Error extends string = DefaultError>(form: FormState<T, State, Error>, name: K): FormField<T, K, State, Error>;
/**
 * Listens for any change on this form. Behaves like useState.
 * You shouldn't use this hook in large components, as it rerenders each time something changes. Use the wrapper <AnyListener /> instead.
 *
 * @param form The form that was passed in.
 */
export declare function useAnyListener<T extends object, State = DefaultState, Error extends string = DefaultError>(form: FormState<T, State, Error>): FormState<T, State, Error>;
/**
 * This is a wrapper around useObjectForm, with useful functions to manipulate arrays.
 * This hook does cause a rerender, but only if the array size changes.
 * @param parentForm The parent form.
 * @param name The parent's field to create a child form for.
 */
export declare function useArrayField<T extends FieldsOfType<any, any[]>, K extends KeysOfType<T, any[] | object>, State = DefaultState, Error extends string = DefaultError>(parentForm: FormState<T, State, Error>, name: K): {
    remove: (index: number) => void;
    move: (from: number, to: number) => void;
    swap: (index: number, newIndex: number) => void;
    clear: () => void;
    append: (value: NonNullable<T[K]>[any]) => void;
    form: ChildFormState<T, K, State, Error>;
    values: NonNullable<T[K]>;
    setValues: (values: Partial<NonNullable<T[K]>> | undefined, validate?: boolean | undefined, isDefault?: boolean | undefined, notifyChild?: boolean, notifyParent?: boolean) => void;
};
/**
 * Listen for truthy changes (if a value becomes truthy or falsy) on a form's field.
 * @param form The form to listen on.
 * @param name The form's field to listen to.
 */
export declare function useTruthyListener<T extends object, K extends keyof T, State = DefaultState, Error extends string = DefaultError>(form: FormState<T, State, Error>, name: K): boolean;
